/*
 * Copyright (c) ISP RAS (http://www.ispras.ru)
 * All Rights Reserved
 *
 * Ivannikov Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
 * 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
 * http://www.ispras.ru
 *
 * The present file was automatically generated on the basis of formal specifications.
 * It is distributed under the same terms and conditions as the derived specifications.
 *
 * N.B. PLEASE DO NOT MODIFY THIS FILE.
 */

package ru.ispras.microtesk.model.mips64.op;

import java.util.Map;
import java.math.BigInteger;
import ru.ispras.microtesk.model.Execution;
import ru.ispras.microtesk.model.ProcessingElement;
import ru.ispras.microtesk.model.data.*;
import ru.ispras.microtesk.model.memory.*;
import ru.ispras.microtesk.model.*;
import ru.ispras.microtesk.model.mips64.PE;
import ru.ispras.microtesk.model.mips64.TempVars;
import ru.ispras.microtesk.model.memory.Location;
import ru.ispras.microtesk.model.mips64.mode.*;

import static ru.ispras.microtesk.model.mips64.TypeDefs.*;

public final class mips64_load extends IsaPrimitive {
  private static final class Info extends IsaPrimitiveInfoAnd {
    Info() {
      super(
          IsaPrimitiveKind.OP,
          "mips64_load",
          mips64_load.class,
          null
          );
      addArgument("load_op", Type.CARD(4));
      addArgument("signed_op", BYTE);
      addArgument("rt", R.INFO);
      addArgument("load_offset", SHORT);
      addArgument("load_base", R.INFO);}

    @Override
    public IsaPrimitive create(final Map<String, IsaPrimitive> args) {
      final Immediate load_op = (Immediate) getArgument("load_op", args);
      final Immediate signed_op = (Immediate) getArgument("signed_op", args);
      final R rt = (R) getArgument("rt", args);
      final Immediate load_offset = (Immediate) getArgument("load_offset", args);
      final R load_base = (R) getArgument("load_base", args);

      return new mips64_load(load_op, signed_op, rt, load_offset, load_base);
    }
  }
  public static final IsaPrimitiveInfoAnd INFO = new Info();

  public final Immediate load_op;
  public final Immediate signed_op;
  public final R rt;
  public final Immediate load_offset;
  public final R load_base;

  public mips64_load(final Immediate load_op, final Immediate signed_op, final R rt, final Immediate load_offset, final R load_base) {
    this.load_op = load_op;
    this.signed_op = signed_op;
    this.rt = rt;
    this.load_offset = load_offset;
    this.load_base = load_base;

    addArgument("load_op", load_op);
    addArgument("signed_op", signed_op);
    addArgument("rt", rt);
    addArgument("load_offset", load_offset);
    addArgument("load_base", load_base);
  }

  @Override
  public void action(final ProcessingElement procElem, final TemporaryVariables tempVars) {
    final PE pe__ = (PE) procElem;
    final TempVars vars__ = (TempVars) tempVars;
    vars__.temp_address_load.access().store(load_base.access(pe__, vars__).load().add(Data.signExtend(DWORD, load_offset.access().load())));
    vars__.load_temp64.access().store(pe__.MEM.access(vars__.temp_address_load.access().load().shiftRight(Data.valueOf(DWORD, 0x3))).load());
    vars__.temp_byte_offset.access().store(vars__.temp_address_load.access().bitField(2, 0).load().xor(Data.valueOf(Type.CARD(3), 0x0)));
    vars__.load_address_error.access().store(Data.zeroExtend(WORD, vars__.temp_byte_offset.access().bitField(2, 0).load()).mod(Data.zeroExtend(WORD, load_op.access().load())));
    if ((vars__.load_address_error.access().load().compareTo(Data.valueOf(WORD, 0x0)) > 0)) {
      pe__.C0_EPC.access().store(pe__.CIA.access().load());
      Execution.exception("AddressError: mips64_load");
      Execution.trace("type = 0x%x", load_op.access().load().bigIntegerValue(false));
    }
    vars__.temp.access().store(Data.valueOf(WORD, 0x8).multiply(Data.zeroExtend(WORD, vars__.temp_byte_offset.access().bitField(2, 0).load())));
    vars__.temp64_0.access().store(vars__.load_temp64.access().load().shiftRight(vars__.temp.access().load()));
    if (signed_op.access().load().equals(Data.valueOf(BYTE, 0x1))) {
      if (load_op.access().load().equals(Data.valueOf(Type.CARD(4), 0x8))) {
        rt.access(pe__, vars__).store(vars__.load_temp64.access().load());
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(4), 0x4))) {
        rt.access(pe__, vars__).store(Data.signExtend(DWORD, vars__.temp64_0.access().bitField(31, 0).load()));
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(4), 0x2))) {
        rt.access(pe__, vars__).store(Data.signExtend(DWORD, vars__.temp64_0.access().bitField(15, 0).load()));
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(4), 0x1))) {
        rt.access(pe__, vars__).store(Data.signExtend(DWORD, vars__.temp64_0.access().bitField(7, 0).load()));
      } else {
        Execution.exception("mips64_load signed: error");
      }
    } else {
      if (load_op.access().load().equals(Data.valueOf(Type.CARD(4), 0x8))) {
        rt.access(pe__, vars__).store(vars__.load_temp64.access().load());
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(4), 0x4))) {
        rt.access(pe__, vars__).store(Data.zeroExtend(DWORD, vars__.temp64_0.access().bitField(31, 0).load()));
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(4), 0x2))) {
        rt.access(pe__, vars__).store(Data.zeroExtend(DWORD, vars__.temp64_0.access().bitField(15, 0).load()));
      } else if (load_op.access().load().equals(Data.valueOf(Type.CARD(4), 0x1))) {
        rt.access(pe__, vars__).store(Data.zeroExtend(DWORD, vars__.temp64_0.access().bitField(7, 0).load()));
      } else {
        Execution.exception("mips64_load unsigned: error");
      }
    }
  }

  @Override
  public void init(final TemporaryVariables tempVars) {
    final PE pe__ = null;
    final TempVars vars__ = (TempVars) tempVars;
  }

  @Override
  public void decode(final TemporaryVariables tempVars) {
    final PE pe__ = null;
    final TempVars vars__ = (TempVars) tempVars;
  }
}